@@ -28,7 +28,7 @@ import subprocess
 import shutil
 import calendar
 
-from fcntl import flock, LOCK_EX, LOCK_UN, LOCK_NB
+from fcntl import lockf, LOCK_EX, LOCK_UN, LOCK_NB
 from StringIO import StringIO
 
 from git import Repo
@@ -36,6 +36,21 @@ from git import Repo
 # default basic logger. We override it later.
 logger = logging.getLogger(__name__)
 
+def fix_remotes(gitdir, toplevel, site):
+    # Remove all existing remotes and set new origin
+    repo = Repo(os.path.join(toplevel, gitdir.lstrip('/')))
+    remotes = repo.git.remote()
+    if len(remotes.strip()):
+        logger.debug('existing remotes: %s' % remotes)
+        for name in remotes.split('\n'):
+            logger.debug('\tremoving remote: %s' % name)
+            repo.git.remote('rm', name)
+
+    # set my origin
+    origin = os.path.join(site, gitdir.lstrip('/'))
+    repo.git.remote('add', '--mirror', 'origin', origin)
+    logger.debug('\tset new origin as %s' % origin)
+
 def set_owner_description(toplevel, gitdir, owner, description):
     if owner is None and description is None:
         # Let the default git values be there, then
@@ -221,7 +236,6 @@ def write_projects_list(manifest, config):
         if os.path.exists(tmpfile):
             os.unlink(tmpfile)
 
-
 def pull_mirror(name, config, opts):
     global logger
     logger = logging.getLogger(name)
@@ -229,7 +243,7 @@ def pull_mirror(name, config, opts):
 
     if 'log' in config.keys():
         ch = logging.FileHandler(config['log'])
-        formatter = logging.Formatter("%(asctime)s - %(levelname)s - %(message)s")
+        formatter = logging.Formatter("[%(process)d] %(asctime)s - %(levelname)s - %(message)s")
         ch.setFormatter(formatter)
         loglevel = logging.INFO
 
@@ -254,72 +268,94 @@ def pull_mirror(name, config, opts):
     # push it into grokmirror to override the default logger
     grokmirror.logger = logger
 
-    logger.info('Updating mirror for [%s]' % name)
-
     # Lock the tree to make sure we only run one instance
     logger.debug('Attempting to obtain lock on %s' % config['lock'])
     flockh = open(config['lock'], 'w')
     try:
-        flock(flockh, LOCK_EX | LOCK_NB)
+        lockf(flockh, LOCK_EX | LOCK_NB)
     except IOError, ex:
-        logger.info('Could not obtain exclusive lock on %s' % config['lock'])
-        logger.info('Assuming another process is running.')
+        logger.debug('Could not obtain exclusive lock on %s' % config['lock'])
+        logger.info('Skipping [%s] - update already running' % name)
         return 0
 
+    logger.info('Checking [%s]' % name)
+
     mymanifest = config['mymanifest']
-    logger.info('Fetching remote manifest from %s' % config['manifest'])
-    request = urllib2.Request(config['manifest'])
-    opener  = urllib2.build_opener()
 
-    # Find out if we need to run at all first
-    if not (opts.force or opts.nomtime) and os.path.exists(mymanifest):
-        fstat = os.stat(mymanifest)
-        mtime = fstat[8]
-        logger.debug('mtime on %s is: %s' % (mymanifest, mtime))
-        my_last_modified = time.strftime('%a, %d %b %Y %H:%M:%S GMT',
-            time.gmtime(mtime))
-        logger.debug('Our last-modified is: %s' % my_last_modified)
-        request.add_header('If-Modified-Since', my_last_modified)
-
-    try:
-        ufh = opener.open(request)
-    except urllib2.HTTPError, ex:
-        if ex.code == 304:
-            logger.info('Server says we have the latest manifest. Quitting.')
-            flock(flockh, LOCK_UN)
+    if config['manifest'].find('file:///') == 0:
+        manifile = config['manifest'].replace('file://', '')
+        fstat = os.stat(manifile)
+        last_modified = fstat[8]
+        fstat = os.stat(config['mymanifest'])
+        my_last_modified = fstat[8]
+        if not (opts.force or opts.nomtime) and last_modified <= my_last_modified:
+            logger.info('Manifest file unchanged. Quitting.')
+            lockf(flockh, LOCK_UN)
             flockh.close()
             return 0
-        logger.critical('Could not fetch %s' % config['manifest'])
-        logger.critical('Server returned: %s' % ex)
-        flock(flockh, LOCK_UN)
-        flockh.close()
-        return 1
-    except urllib2.URLError, ex:
-        logger.critical('Could not fetch %s' % config['manifest'])
-        logger.critical('Error was: %s' % ex)
-        flock(flockh, LOCK_UN)
-        flockh.close()
-        return 1
 
-    last_modified = ufh.headers.get('Last-Modified')
-    last_modified = time.strptime(last_modified, '%a, %d %b %Y %H:%M:%S %Z')
-    last_modified = calendar.timegm(last_modified)
+        logger.info('Reading new manifest from %s' % manifile)
+        manifest = grokmirror.read_manifest(manifile)
+        # Don't accept empty manifests -- that indicates something is wrong
+        if not len(manifest.keys()):
+            logger.critical('Remote manifest empty or unparseable! Quitting.')
+            return 1
 
-    # We don't use read_manifest for the remote manifest, as it can be
-    # anything, really. For now, blindly open it with gzipfile if it ends
-    # with .gz. XXX: some http servers will auto-deflate such files.
-    if config['manifest'].find('.gz') > 0:
-        fh = gzip.GzipFile(fileobj=StringIO(ufh.read()))
     else:
-        fh = ufh
+        # Load it from remote host using http and header magic
+        logger.info('Fetching remote manifest from %s' % config['manifest'])
+        request = urllib2.Request(config['manifest'])
+        opener  = urllib2.build_opener()
 
-    try:
-        manifest = json.load(fh)
-    except:
-        logger.critical('Failed to parse %s' % config['manifest'])
-        flock(flockh, LOCK_UN)
-        flockh.close()
-        return 1
+        # Find out if we need to run at all first
+        if not (opts.force or opts.nomtime) and os.path.exists(mymanifest):
+            fstat = os.stat(mymanifest)
+            mtime = fstat[8]
+            logger.debug('mtime on %s is: %s' % (mymanifest, mtime))
+            my_last_modified = time.strftime('%a, %d %b %Y %H:%M:%S GMT',
+                time.gmtime(mtime))
+            logger.debug('Our last-modified is: %s' % my_last_modified)
+            request.add_header('If-Modified-Since', my_last_modified)
+
+        try:
+            ufh = opener.open(request)
+        except urllib2.HTTPError, ex:
+            if ex.code == 304:
+                logger.info('Server says we have the latest manifest. Quitting.')
+                lockf(flockh, LOCK_UN)
+                flockh.close()
+                return 0
+            logger.critical('Could not fetch %s' % config['manifest'])
+            logger.critical('Server returned: %s' % ex)
+            lockf(flockh, LOCK_UN)
+            flockh.close()
+            return 1
+        except urllib2.URLError, ex:
+            logger.critical('Could not fetch %s' % config['manifest'])
+            logger.critical('Error was: %s' % ex)
+            lockf(flockh, LOCK_UN)
+            flockh.close()
+            return 1
+
+        last_modified = ufh.headers.get('Last-Modified')
+        last_modified = time.strptime(last_modified, '%a, %d %b %Y %H:%M:%S %Z')
+        last_modified = calendar.timegm(last_modified)
+
+        # We don't use read_manifest for the remote manifest, as it can be
+        # anything, really. For now, blindly open it with gzipfile if it ends
+        # with .gz. XXX: some http servers will auto-deflate such files.
+        if config['manifest'].find('.gz') > 0:
+            fh = gzip.GzipFile(fileobj=StringIO(ufh.read()))
+        else:
+            fh = ufh
+
+        try:
+            manifest = json.load(fh)
+        except:
+            logger.critical('Failed to parse %s' % config['manifest'])
+            lockf(flockh, LOCK_UN)
+            flockh.close()
+            return 1
 
     mymanifest = grokmirror.read_manifest(mymanifest)
 
@@ -349,10 +385,15 @@ def pull_mirror(name, config, opts):
     existing = []
 
     toplevel = config['toplevel']
+    if not os.access(toplevel, os.W_OK):
+        logger.critical('Toplevel %s does not exist or is not writable' % toplevel)
+        sys.exit(1)
+
     for gitdir in culled.keys():
+        fullpath = os.path.join(toplevel, gitdir.lstrip('/'))
         if gitdir in mymanifest.keys():
             # Is the directory in place, too?
-            if os.path.exists(os.path.join(toplevel, gitdir.lstrip('/'))):
+            if os.path.exists(fullpath):
                 # Is it newer than what we have in our old manifest?
                 if gitdir in mymanifest.keys():
                     # This code is hurky and needs to be cleaned up
@@ -377,8 +418,14 @@ def pull_mirror(name, config, opts):
                 continue
 
         # do we have the dir in place?
-        if os.path.exists(os.path.join(toplevel, gitdir.lstrip('/'))):
-            # blindly assume it's kosher and pull it
+        if os.path.exists(fullpath):
+            if not opts.reuse:
+                logger.critical('Found existing repository in %s' % fullpath)
+                logger.critical('Run with -r to use existing repos')
+                sys.exit(1)
+            logger.info('Found existing %s, will set new origin' % gitdir)
+            # Accept it, but fix remotes so they are pointing to our origin
+            fix_remotes(gitdir, toplevel, config['site'])
             to_pull.append(gitdir)
             continue
 
@@ -394,6 +441,7 @@ def pull_mirror(name, config, opts):
             pull_repo(toplevel, gitdir)
             set_agefile(toplevel, gitdir, culled[gitdir]['modified'])
             run_post_update_hook(hookscript, toplevel, gitdir)
+            grokmirror.unlock_repo(fullpath)
         except IOError, ex:
             logger.info('Could not obtain exclusive lock on %s' % gitdir)
             logger.info('\tAssuming grok-fsck is running, will try later.')
@@ -442,7 +490,7 @@ def pull_mirror(name, config, opts):
                 if symlink not in symlinks:
                     symlinks.append(symlink)
                 target = os.path.join(config['toplevel'], symlink.lstrip('/'))
-                if not os.path.exists(target):
+                if not os.path.exists(target) and os.path.exists(source):
                     logger.info('Symlinking %s -> %s' % (target, source))
                     # Make sure the leading dirs are in place
                     if not os.path.exists(os.path.dirname(target)):
@@ -453,9 +501,9 @@ def pull_mirror(name, config, opts):
         for founddir in grokmirror.find_all_gitdirs(config['toplevel']):
             gitdir = founddir.replace(config['toplevel'], '')
             if gitdir not in culled.keys() and gitdir not in symlinks:
-                if os.path.islink(gitdir):
+                if os.path.islink(founddir):
                     logger.info('Removing unreferenced symlink %s' % gitdir)
-                    os.unlink(gitdir)
+                    os.unlink(founddir)
                 else:
                     logger.info('Purging %s' % gitdir)
                     shutil.rmtree(founddir)
@@ -467,7 +515,7 @@ def pull_mirror(name, config, opts):
     write_projects_list(culled, config)
 
     logger.debug('Unlocking %s' % config['lock'])
-    flock(flockh, LOCK_UN)
+    lockf(flockh, LOCK_UN)
     flockh.close()
 
     return 127
@@ -495,6 +543,10 @@ if __name__ == '__main__':
     parser.add_option('-p', '--purge', dest='purge',
         action='store_true', default=False,
         help='Remove any git trees that are no longer in manifest.')
+    parser.add_option('-r', '--reuse-existing-repos', dest='reuse',
+        action='store_true', default=False,
+        help='If any existing repositories are found on disk, set new '
+        'remote origin and reuse')
     parser.add_option('-c', '--config', dest='config',
         help='Location of repos.conf')
 
